public class BST {
  private class Node {
    private String value; // not null, for simplicity
    private Node left;
    private Node right;
    
    /** constructor: a node with value val
      * precondition: val is not null */
    public Node(String val) {
      value= val;
    }
  }
  
  private Node root; // root node of the tree; may be null
  
  public BST() { };
  
  /** true iff val is in the tree */
  public boolean contains(String val) {
    return contains(root, val);
  }
  
  /** true if val is in the subtree rooted at n */
  private boolean contains(Node n, String val) {
    if (n == null || val == null)
      return false;
    int compare = n.value.compareTo(val);
    if (compare == 0)
      return true;
    if (compare > 0)
      return contains(n.right, val);
    // {compare < 0}
    return contains(n.left, val);
  }
  
  /** insert val into the tree 
    * pre: val is not null */
  public void insert(String val) {
    root = insert(root, val);
  }
  /** insert val into the subtree rooted at node,
    * returning the root of the modified subtree
    * pre: val is not null */
  private Node insert(Node node, String val) {
    if (node == null) 
      return new Node(val);
    
    int compare = node.value.compareTo(val);
    
    if (compare == 0) // it was here all along!
      return node; 

    if (compare > 0) // it belongs in node's left subtree
      node.left = insert(node.left, val);
    else // it belongs in the node's right subtree
      node.right = insert(node.right, val);
    
    return node;
  }   
  
  /** print the elements in the tree in sorted order */
  public void printInOrder() {
    printInOrder(root);
    System.out.println();
  }
  private void printInOrder(Node n) {
    if (n == null) return;
    if (n.left != null) printInOrder(n.left);
    System.out.print(n.value + " ");
    if (n.right != null) printInOrder(n.right);
  }
   
  
}